"""await-ccb

Given a git SHA, waits for a corresponding Google Cloud Container Builder build
to complete successfully.

- If a successful build exists, exits 0.
- If a running or queued build exists, polls until no running builds are found
  or the --poll-limit is reached.
- If no running/queued build is found in the initial search, repeats the search
  up to --search-limit times to see if a running build appears.

Usage:
    await-ccb -c credentials.json -r repo-name -s git-sha
        [-t -p my-project --poll-interval 5 --poll-limit 360 --search-limit 5]
    await-ccb --help
    await-ccb --version

Options:
    -c CREDENTIALS_JSON, --credentials CREDENTIALS_JSON
        Path to service account credentials in Google JSON format

    -p PROJECT, --project PROJECT
        Google Code Project ID, defaults to project from credentials

    -r REPO, --repo REPO
        Repo name, as known to CCB (e.g. github-thedyrt-await-ccb)

    -s SHA, --sha SHA
        Git SHA to observer builds for

    -t, --trigger
        Trigger a build if none are found. Requires an existing build trigger
        to be configured for the repo in the Google Cloud Console.

    --search-limit LIMIT
        Number of attempts to make in the initial search for running builds
        [Default: 3]

    --poll-limit LIMIT
        Number of poll requests to make before giving waiting on running builds
        to complete. Total polling time is --poll-limit * --poll-interval.
        [Default: 360]

    --poll-interval SECONDS
        Number of seconds to wait between requests
        [Default: 5]

    -h, --help
        Show this help

    --version
        Show version info
"""
from docopt import docopt
from sys import exit

from .build_poller import BuildPoller


def main():
    arguments = docopt(__doc__, version='await-ccb 0.1.0')
    print(arguments)

    poller = BuildPoller(
        credentials_path=arguments['--credentials'],
        project=arguments['--project'],
        repo=arguments['--repo'],
        sha=arguments['--sha'],
        trigger=arguments['--trigger'],
        search_limit=int(arguments['--search-limit']),
        poll_limit=int(arguments['--poll-limit']),
        poll_interval=int(arguments['--poll-interval']),
    )

    if poller.await_success():
        exit(0)
    else:
        exit(1)



from google.oauth2 import service_account
from time import sleep
import googleapiclient.discovery

class BuildPoller:
    INCOMPLETE_STATES = ('QUEUED', 'WORKING')
    SUCCESSS_STATES = ('SUCCESS',)

    def __init__(self, credentials_path, project, repo, sha, trigger,
                 search_limit=3, poll_limit=360, poll_interval=5):
        self.credentials = self.__load_credentials(credentials_path)
        self.project = project or self.credentials.project_id
        self.search_limit = search_limit
        self.poll_limit = poll_limit
        self.poll_interval = poll_interval
        self.repo = repo
        self.sha = sha
        self.trigger_build = trigger
        self.client = googleapiclient.discovery.build(
                        'cloudbuild', 'v1', credentials=self.credentials)

    def __load_credentials(self, credentials_path):
        return service_account.Credentials.from_service_account_file(
                credentials_path,
                scopes=['https://www.googleapis.com/auth/cloud-platform'])

    def await_success(self):
        builds = []

        def any_successful():
            return any([b['status'] in self.SUCCESSS_STATES for b in builds])

        def any_incomplete():
            return any([b['status'] in self.INCOMPLETE_STATES for b in builds])

        for search_attempt in range(1, self.search_limit):
            builds = self.builds_for_commit()
            if any_incomplete() or any_successful():
                break

            sleep(self.poll_interval)

        if self.trigger_build and not (any_successful() or any_incomplete()):
            with self.run_trigger() as build:
                if build:
                    builds = [build]

        for poll_attempt in range(1, self.poll_limit):
            if not any_incomplete():
                break

            sleep(self.poll_interval)
            builds = self.builds_for_commit()

        return any_successful()

    def run_trigger(self):
        trigger = self.repo_trigger()

        if not trigger:
            return None

        result = (
            self.client
            .projects()
            .triggers()
            .run(
                projectId=self.project,
                triggerId=trigger['id'],
                body={
                    'projectId': self.project,
                    'repoName': self.repo,
                    'commitSha': self.sha,
                })
            .execute())

        if 'metadata' in result and 'build' in result['metadata']:
            return result['metadata']['build']
        else:
            return None

    def repo_trigger(self):
        result = (
            self.client
            .projects()
            .triggers()
            .list(projectId=self.project)
            .execute())

        if 'triggers' in result:
            trigger = [
                t for t in result['triggers']
                if t['triggerTemplate']['repoName'] == self.repo
            ]

            if trigger:
                return trigger[0]

    def builds_for_commit(self):
        return self.find_builds((
            'source.repo_source.repo_name="{0}"'
            ' AND source_provenance.resolved_repo_source.commit_sha="{1}"'
            ).format(self.repo, self.sha))

    def find_builds(self, query):
        result = self.client.projects().builds().list(
            projectId=self.project,
            filter=query
        ).execute()

        if 'builds' in result:
            return result['builds']
        else:
            return []



from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='await_ccb',
    version='0.1.0',
    description='Waits for Google Cloud Container Build to complete',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Reid Beels',
    author_email='reid@thedyrt.com',
    url='https://github.com/thedyrt/await-ccb',
    license='MIT',
    keywords='google cloud container builder',
    packages=find_packages(),
    install_requires=[
        'google-auth',
        'google-auth-httplib2',
        'google-api-python-client',
        'docopt'
    ],
    entry_points={
        'console_scripts': ['await-ccb=await_ccb.cli:main']
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6'
    ],
)
